@@ -3,7 +3,9 @@ from __future__ import annotations
 from ._df_helper import DfHelper
 from ._parquet_artifact import ParquetArtifact
 from ._parquet_reader import ParquetReader
-from ._artifact_updater_multi_wrapper import ArtifactUpdaterMultiWrapperThreaded, ArtifactUpdaterMultiWrapperAsync
+#from ._artifact_updater_multi_wrapper import ArtifactUpdaterMultiWrapperThreaded, ArtifactUpdaterMultiWrapperAsync
+from ._artifact_updater_async import ArtifactUpdaterMultiWrapperAsync
+from ._artifact_updater_threaded import ArtifactUpdaterMultiWrapperThreaded
 
 __all__ = [
     'DfHelper',
