@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import math
 import os
 import pickle
@@ -9,236 +11,102 @@ import numpy as np
 import osmnx as ox
 from geopy.distance import geodesic
 
-
-#
-# options = {
-#    'ox_files_save_path': ox_files_save_path,
-#    'network_type': 'drive',
-#    'place': 'Costa Rica',
-#    'files_prefix': 'costa-rica-',
-# }
-# Usage example
-# handler = PBFHandler(**options)
-# handler.load()
-
+from typing import Optional
+from fsspec.core import url_to_fs
 
 class PBFHandler:
     """
-    Handles the creation, management, and visualization of graph data derived
-    from .pbf (Protocolbuffer Binary Format) files. This class enables the
-    loading, processing, saving, and reutilization of graph, node, and edge
-    data for geographical regions, supporting verbose mode for detailed outputs.
-
-    :ivar graph: The generated graph object representing the spatial network; can be None if not yet loaded or processed.
-    :type graph: Optional[NetworkX.Graph]
-    :ivar nodes: GeoDataFrame representing the nodes of the graph; can be None if not yet loaded or processed.
-    :type nodes: Optional[geopandas.GeoDataFrame]
-    :ivar edges: GeoDataFrame representing the edges of the graph; can be None if not yet loaded or processed.
-    :type edges: Optional[geopandas.GeoDataFrame]
-    :ivar rebuild: Indicates whether to rebuild the graph data, ignoring any existing cached files. Default is ``False``.
-    :type rebuild: bool
-    :ivar verbose: Enables verbose mode to provide detailed status messages during operations. Default is ``False``.
-    :type verbose: bool
-    :ivar place: The name of the geographical region to process with OpenStreetMap. Default is ``Costa Rica``.
-    :type place: str
-    :ivar filepath: The path to the directory where the graph, nodes, and edges pickle files are saved. Default is ``gis_data/``.
-    :type filepath: str
-    :ivar file_prefix: The prefix for the filenames of the saved graph, node, and edge pickle files. Default is ``costa-rica-``.
-    :type file_prefix: str
-    :ivar network_type: The type of network to extract from OpenStreetMap, such as "all" or other specific network types. Default is ``all``.
-    :type network_type: str
-    :ivar graph_file: Full path of the file to save or load the graph data as a pickle file.
-    :type graph_file: str
-    :ivar node_file: Full path of the file to save or load the graph's node data as a pickle file.
-    :type node_file: str
-    :ivar edge_file: Full path of the file to save or load the graph's edge data as a pickle file.
-    :type edge_file: str
+    Build/load OSMnx graph + nodes/edges; persist as pickle via fsspec.
     """
+
     def __init__(self, **kwargs):
         self.graph = None
-        self.nodes = None
-        self.edges = None
-        self.rebuild = kwargs.setdefault("rebuild", False)
-        self.verbose = kwargs.setdefault("verbose", False)
-        self.place = kwargs.setdefault('place', 'Costa Rica')
-        self.filepath = kwargs.setdefault('ox_files_save_path', "gis_data/")
-        self.file_prefix = kwargs.setdefault('file_prefix', 'costa-rica-')
-        self.network_type = kwargs.setdefault('network_type', 'all')
-        self.graph_file = f"{self.filepath}{self.file_prefix}graph.pkl"
-        self.node_file = f"{self.filepath}{self.file_prefix}nodes.pkl"
-        self.edge_file = f"{self.filepath}{self.file_prefix}edges.pkl"
+        self.nodes: Optional[gpd.GeoDataFrame] = None
+        self.edges: Optional[gpd.GeoDataFrame] = None
 
-    def load(self):
-        """
-        Loads the required data files for processing. If the files do not exist or
-        if the `rebuild` flag is set to True, it will process and recreate the
-        necessary data from the source. Otherwise, it will load the data from
-        existing pickle files. This function ensures the target directory exists,
-        and processes files conditionally based on their presence.
+        self.rebuild: bool = kwargs.setdefault("rebuild", False)
+        self.verbose: bool = kwargs.setdefault("verbose", False)
+        self.place: str = kwargs.setdefault("place", "Costa Rica")
+        self.network_type: str = kwargs.setdefault("network_type", "all")
+        base_url: str = kwargs.setdefault("data_path", "osmnx_data/pbf_files")
+        prefix: str = kwargs.setdefault("files_prefix", "costa-rica-").rstrip("-") + "-"
 
-        :param verbose: Flag to control the verbosity of the function's output.
-        :param rebuild: Indicates whether the data should be rebuilt from the raw
-            source files.
-        :param graph_file: Path to the graph file to be loaded or rebuilt.
-        :param node_file: Path to the node file to be loaded or rebuilt.
-        :param edge_file: Path to the edge file to be loaded or rebuilt.
-        :param filepath: Path to the directory where files are processed and saved.
+        # Allow passing an fsspec instance directly
+        fs = kwargs.get("fs")
+        if fs is not None:
+            self.fs = fs
+            self.base = base_url.rstrip("/")
+        else:
+            self.fs, self.base = url_to_fs(base_url)
+
+        self.fs.mkdirs(self.base, exist_ok=True)
+
+        self.graph_file = f"{self.base.rstrip('/')}/{prefix}graph.pkl"
+        self.node_file = f"{self.base.rstrip('/')}/{prefix}nodes.pkl"
+        self.edge_file = f"{self.base.rstrip('/')}/{prefix}edges.pkl"
 
-        :return: None
-        """
         if self.verbose:
-            print("Loading data...")
+            print(f"[PBFHandler] base={self.base}")
+            print(f"  graph={self.graph_file}")
+            print(f"  nodes={self.node_file}")
+            print(f"  edges={self.edge_file}")
 
-        files_to_check = [self.graph_file, self.node_file, self.edge_file]
+    # ---------- public API ----------
+    def load(self) -> None:
+        if self.verbose:
+            print("[PBFHandler] load()")
 
         if self.rebuild:
-            for file in files_to_check:
-                if os.path.exists(file):
-                    os.remove(file)
-        if not os.path.exists(self.filepath):
-            os.makedirs(self.filepath, exist_ok=True)
-            # self.process_pbf()
-            # self.save_to_pickle()
-        if not all(os.path.exists(f) for f in files_to_check):
+            self._delete_artifacts()
+
+        if not self._artifacts_exist():
             self.process_pbf()
             self.save_to_pickle()
         else:
             self.load_from_pickle()
 
+    def process_pbf(self) -> None:
         if self.verbose:
-            print("Data loaded successfully.")
+            print(f"[PBFHandler] processing: {self.place}")
+        self.graph = ox.graph_from_place(self.place, network_type=self.network_type)
+        self.nodes, self.edges = ox.graph_to_gdfs(self.graph)
 
-    def process_pbf(self):
-        """
-        Processes the Protocolbuffer Binary Format (PBF) data specified for a given place by
-        utilizing the OSMnx library to create a graph representation and extracts nodes and
-        edges into GeoDataFrames. The function provides verbose output if enabled.
+    def save_to_pickle(self) -> None:
+        if self.verbose:
+            print("[PBFHandler] saving via fsspec")
+        for path, obj in {
+            self.graph_file: self.graph,
+            self.node_file: self.nodes,
+            self.edge_file: self.edges,
+        }.items():
+            if obj is not None:
+                with self.fs.open(path, "wb") as f:
+                    pickle.dump(obj, f, protocol=pickle.HIGHEST_PROTOCOL)
 
-        :param self: Refers to the current instance of the class containing this method.
+    def load_from_pickle(self) -> None:
+        if self.verbose:
+            print("[PBFHandler] loading via fsspec")
+        self.graph = self._load_pickle(self.graph_file)
+        self.nodes = self._load_pickle(self.node_file)
+        self.edges = self._load_pickle(self.edge_file)
 
-        :param self.verbose: bool
-            A flag to control verbose output. If True, detailed processing status messages are
-            logged to the console.
+    # ---------- helpers ----------
+    def _artifacts_exist(self) -> bool:
+        return all(self.fs.exists(p) for p in (self.graph_file, self.node_file, self.edge_file))
 
-        :param self.place: str
-            The name or description of the geographic place for which PBF data is processed. It
-            is used to construct a graph representation of the place.
+    def _delete_artifacts(self) -> None:
+        if self.verbose:
+            print("[PBFHandler] deleting artifacts (rebuild=True)")
+        for p in (self.graph_file, self.node_file, self.edge_file):
+            if self.fs.exists(p):
+                try:
+                    self.fs.rm_file(p)
+                except Exception:
+                    self.fs.rm(p)
 
-        :param self.network_type: str
-            The type of network graph to be created, typically one of 'all', 'walk', 'drive',
-            etc., reflecting the type of paths or streets included in the graph.
-
-        :return: None
-            This function does not return a value, but updates class attributes ``graph``,
-            ``nodes``, and ``edges``.
-
-        :raises Exception:
-            Raises a general exception when there is an error in processing the PBF data. Error
-            details are printed when verbose output is enabled.
-        """
-        try:
-            if self.verbose:
-                print(f"Processing PBF for {self.place}...")
-
-            self.graph = ox.graph_from_place(self.place, network_type=self.network_type)
-            self.nodes, self.edges = ox.graph_to_gdfs(self.graph)
-
-            if self.verbose:
-                print("PBF processed successfully.")
-        except Exception as e:
-            print(f"Error processing PBF: {e}")
-            raise
-
-    def save_to_pickle(self):
-        """
-        Saves data, including graph, nodes, and edges, to pickle files. Each data object is
-        saved to its corresponding file if available. If verbose mode is enabled, prints
-        messages indicating the saving progress and success.
-
-        :param self:
-            Represents the instance of the class that contains attributes `graph_file`,
-            `graph`, `node_file`, `nodes`, `edge_file`, `edges`, and `verbose`. These
-            attributes determine the files to save to and the data to save.
-
-        :raises Exception:
-            Raises an exception if an error occurs during the saving process.
-
-        :return:
-            None
-        """
-        try:
-            if self.verbose:
-                print("Saving data to pickle files...")
-
-            data_to_save = {
-                self.graph_file: self.graph,
-                self.node_file: self.nodes,
-                self.edge_file: self.edges
-            }
-
-            for file, data in data_to_save.items():
-                if data is not None:
-                    with open(file, 'wb') as f:
-                        pickle.dump(data, f, pickle.HIGHEST_PROTOCOL)
-
-            if self.verbose:
-                print("Data saved to pickle files successfully.")
-        except Exception as e:
-            print(f"Error saving to pickle: {e}")
-            raise
-
-    def load_from_pickle(self):
-        """
-        Loads data from pickle files specified by the attributes `graph_file`, `node_file`,
-        and `edge_file` and assigns them to the corresponding attributes `graph`,
-        `nodes`, and `edges`, respectively. Displays verbose messages during the load
-        process if the `verbose` attribute is set to True.
-
-        :raises Exception: If an error occurs during reading or deserialization of the
-                           pickle files.
-        """
-        try:
-            if self.verbose:
-                print("Loading data from pickle files...")
-
-            files_to_load = {
-                self.graph_file: 'graph',
-                self.node_file: 'nodes',
-                self.edge_file: 'edges'
-            }
-
-            for file, attr in files_to_load.items():
-                with open(file, 'rb') as f:
-                    setattr(self, attr, pickle.load(f))
-
-            if self.verbose:
-                print("Data loaded from pickle files successfully.")
-        except Exception as e:
-            print(f"Error loading from pickle: {e}")
-            raise
-
-    def plot_graph(self):
-        """
-        Plots the loaded graph using the OSMnx library.
-
-        This method checks if a graph is loaded and, if available, plots it. Outputs
-        verbose messages during the process if verbosity is enabled.
-
-        :raises Exception: Raises if an error occurs during the plotting process.
-        :return: None
-        """
-        try:
-            if self.graph is not None:
-                if self.verbose:
-                    print("Plotting the graph...")
-                ox.plot_graph(self.graph)
-                if self.verbose:
-                    print("Graph plotted successfully.")
-            else:
-                print("Graph is not loaded. Please load a PBF file first.")
-        except Exception as e:
-            print(f"Error plotting the graph: {e}")
-            raise
+    def _load_pickle(self, path: str):
+        with self.fs.open(path, "rb") as f:
+            return pickle.load(f)
 
 
 def get_bounding_box_from_points(gps_points, margin=0.001):
